@@ -1,5 +1,8 @@
 package Gherkin::Parser;
-$Gherkin::Parser::VERSION = '17.0.2';
+$Gherkin::Parser::VERSION = '18.0.0';
+use strict;
+use warnings;
+
 use base 'Gherkin::Generated::Parser';
 
 1;
