@@ -1,12 +1,13 @@
 package Gherkin::Pickles::Compiler;
-$Gherkin::Pickles::Compiler::VERSION = '17.0.2';
+$Gherkin::Pickles::Compiler::VERSION = '18.0.0';
 use strict;
 use warnings;
 use Scalar::Util qw(reftype);
 
 sub compile {
-    my ( $class, $gherkin_document, $id_generator ) = @_;
+    my ( $class, $gherkin_document, $id_generator, $pickle_sink ) = @_;
     my @pickles;
+    $pickle_sink ||= sub { push @pickles, $_[0] };
 
     my $feature = $gherkin_document->{'gherkinDocument'}->{'feature'};
     my $language         = $feature->{'language'};
@@ -21,14 +22,14 @@ sub compile {
                 $gherkin_document->{'gherkinDocument'}->{'uri'},
                 $feature_tags, $background_steps,
                 $child->{'scenario'}, $language, $id_generator,
-                \@pickles
+                $pickle_sink
                 );
         } elsif ( $child->{'rule'} ) {
             $class->_compile_rule(
                 $gherkin_document->{'gherkinDocument'}->{'uri'},
                 $feature_tags, $background_steps,
                 $child->{'rule'}, $language, $id_generator,
-                \@pickles
+                $pickle_sink
                 );
         } else {
             ...;
@@ -74,7 +75,7 @@ sub reject_nones {
 
 sub _compile_scenario {
     my ( $class, $uri, $feature_tags, $background_steps,
-         $scenario, $language, $id_generator, $pickles )
+         $scenario, $language, $id_generator, $pickle_sink )
         = @_;
 
     for my $examples ( @{ $scenario->{'examples'} ||
@@ -119,8 +120,7 @@ sub _compile_scenario {
                 }
             }
 
-            push(
-                @$pickles,
+            $pickle_sink->(
                 {
                     pickle =>
                         reject_nones(
@@ -141,17 +141,20 @@ sub _compile_scenario {
                                      $values->{'id'}
                                     ])
                             })
-                }
-            );
+                });
         }
     }
 }
 
 sub _compile_rule {
     my ( $class, $uri, $feature_tags, $feature_background_steps,
-         $rule_definition, $language, $id_generator, $pickles )
+         $rule_definition, $language, $id_generator, $pickle_sink )
         = @_;
     my $background_steps = $feature_background_steps;
+    my @tags = (
+        @{ $feature_tags || [] },
+        @{ $rule_definition->{'tags'} || []}
+    );
 
     for my $child ( @{ $rule_definition->{'children'} } ) {
         if ( $child->{'background'} ) {
@@ -160,9 +163,9 @@ sub _compile_rule {
                   @{ $child->{'background'}->{'steps'} } ];
         } elsif ( $child->{'scenario'} ) {
             $class->_compile_scenario(
-                $uri, $feature_tags, $background_steps,
+                $uri, \@tags, $background_steps,
                 $child->{'scenario'}, $language, $id_generator,
-                $pickles
+                $pickle_sink
                 );
         } else {
             ...;
